package HoursToolWeb::Controller::Admin::LocationHours;
use Moose;
use namespace::autoclean;

use String::Util qw(hascontent);

BEGIN { extends 'Catalyst::Controller'; }

=head1 NAME

HoursToolWeb::Controller::Admin::LocationHours - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

=cut

sub base :Chained('../base') :PathPart('location_hours') :CaptureArgs(0) {
    my ( $self, $c ) = @_;
    $c->stash->{section} = 'location_hours';
}


=head2 index

=cut

sub index :Chained('base') :PathPart('') :Args(0) {
    my ( $self, $c ) = @_;

    $c->form({
            optional => [ qw( show filter apply_filter sort page ) ],
            filters  => ['trim'],
    });

    my %sort_map = (
        location        => 'LOWER(location.name)',
        location_d      => 'LOWER(location.name) DESC',
        range           => 'LOWER(range.name)',
        range_d         => 'LOWER(range.name) DESC',
    );

    my $lh_rs = $c->model('HoursToolDB::LocationHours')->search({},
        {
            order_by  => $sort_map{$c->form->valid('sort') || 'name'},
            page      => int( $c->form->valid('page') || 1 ),
            rows      => 30,
            prefetch  => [ 'location', 'range' ],
        }
    );

    if ( my $filter = $c->form->valid('filter') ) {
        $filter =~ s/[%_]/\\$1/g;
        $lh_rs = $lh_rs->search({
        	'-or' => {
        		'location.name' => { 'ilike' => "%${filter}%" },
	        	'range.name' 	=> { 'ilike' => "%${filter}%" },
	        },
        });
    }

    $c->stash->{page}         = $c->req->params->{page};
    $c->stash->{sort}         = $c->req->params->{sort} || 'location';
    $c->stash->{filter}       = $c->req->params->{filter};
    $c->stash->{lh_rs} 		  = $lh_rs;
    $c->stash->{template}     = 'admin/location_hours/list.tt';
}


sub edit :Chained('base') :PathPart('edit') :Args(1) {
    my ($self, $c, $lh_id) = @_;

    $c->form({
        required               => [ qw( location_id range_id priority active ) ],
        optional               => [ qw( submit ) ],
        optional_regexp        => qr/^\d+\-/,
        filters                => [ qw( trim ) ],
        missing_optional_valid => 1,
    });

    my $lh;
    if ( $lh_id && $lh_id ne 'new' ) {
        $lh = $c->model('HoursToolDB::LocationHours')->find({ id => $lh_id });
    }

    if ( hascontent($c->form->valid->{submit}) ) {
        
        $c->stash->{form_submitted} = 1;
        $c->stash->{params} = $c->request->params;

        unless ($c->form->has_missing || $c->form->has_invalid || $c->form->has_unknown) {

            my $redirect = 0;
            eval {

                $c->model('HoursToolDB')->txn_do( sub {

                    if ( $lh ) {
                        $lh->update_from_fv($c->form);

                        foreach my $dow ( 1 .. 7 ) {
                            my $open_time       = $c->form->valid("${dow}-open_time");
                            my $close_time      = $c->form->valid("${dow}-close_time");
                            my $closed_all_day  = $c->form->valid("${dow}-closed_all_day");
                            $lh->update_dow( $dow, { open_time => $open_time, close_time => $close_time, closed_all_day => $closed_all_day ? 't' : 'f' })
                        }

                    }
                    else {
                        
                        # Create main LocationHours record

                        $lh = $c->model('HoursToolDB::LocationHours')->create_from_fv($c->form);

                        # Create attached day records

                        foreach my $day ( 1 .. 7 ) {
                            $lh->add_to_days({ dow => $day, });
                        }

                        $c->flash->{results} = [ $c->loc('Location Hours created.') ];
                        $redirect = 1;
                    }

                });
            };
            if ($@) {
                my $err = $@;
                die($err);
            }

            if ( $redirect ) {
                $c->redirect( $c->uri_for( $c->controller->action_for('edit'), [ $lh->id ] ) );
            }

            push @{$c->stash->{results}}, $c->loc('Location Hours updated.');
        }

    }

    $c->stash->{location_hours} = $lh;
    $c->stash->{locations}      = [ $c->model('HoursToolDB::Locations')->all ];
    $c->stash->{ranges}         = [ $c->model('HoursToolDB::Ranges')->all ];
    $c->stash->{template}       = 'admin/location_hours/edit.tt';
}


sub delete :Chained('base') :PathPart('delete') :Args(1) {
    my ($self, $c, $lh_id) = @_;
    
    my $lh = $c->model('HoursToolDB::LocationHours')->find({ id => $lh_id });
    if ( !defined($lh) ) {
        die( $c->loc('Unable to find location hours id: ') . $lh_id );
        $c->detach;
        return;
    }

    if ( $c->req->params->{do_delete} ) {
        # TODO: Check that this is chain deleting all the attached records
        
        $lh->delete();
        
        return $c->redirect( $c->uri_for( $c->controller->action_for('index') ) );
    }

    $c->stash->{location_hours} = $lh;
    $c->stash->{template}       = 'admin/location_hours/delete.tt';
}

sub view :Chained('base') :PathPart('view') :Args(1) {
    my ($self, $c, $lh_id) = @_;
    
    my $lh = $c->model('HoursToolDB::LocationHours')->find({ id => $lh_id });
    if ( !defined($lh) ) {
        die( $c->loc('Unable to find location hours id: ') . $lh_id );
        $c->detach;
        return;
    }

    $c->stash->{location_hours} = $lh;
    $c->stash->{template}       = 'admin/location_hours/view.tt';
}




=head1 AUTHOR

Todd Holbrook

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
