package HoursToolWeb::Controller::Admin::Locations;
use Moose;
use namespace::autoclean;

use String::Util qw(hascontent);

BEGIN { extends 'Catalyst::Controller'; }

=head1 NAME

HoursToolWeb::Controller::Admin::Locations - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

=cut

sub base :Chained('../base') :PathPart('location') :CaptureArgs(0) {
    my ( $self, $c ) = @_;
    $c->stash->{section} = 'locations';
}


=head2 index

=cut

sub index :Chained('base') :PathPart('') :Args(0) {
    my ( $self, $c ) = @_;

    $c->form({
            optional => [ qw( show filter apply_filter sort page ) ],
            filters  => ['trim'],
    });

    my %sort_map = (
        name        => 'LOWER(name)',
        name_d      => 'LOWER(name) DESC',
        rank        => 'rank, LOWER(name)',
        rank_d      => 'rank DESC, LOWER(name)',
    );

    my $locations_rs = $c->model('HoursToolDB::Locations')->search({},
        {
            order_by  => $sort_map{$c->form->valid('sort') || 'name'},
            page      => int( $c->form->valid('page') || 1 ),
            rows      => 30,
        }
    );

    if ( my $filter = $c->form->valid('filter') ) {
        $filter =~ s/[%_]/\\$1/g;
        $locations_rs = $locations_rs->search({ name => { 'ilike' => "%${filter}%" } });
    }

    $c->stash->{page}         = $c->req->params->{page};
    $c->stash->{sort}         = $c->req->params->{sort} || 'name';
    $c->stash->{filter}       = $c->req->params->{filter};
    $c->stash->{locations_rs} = $locations_rs;
    $c->stash->{template}     = 'admin/locations/list.tt';
}


sub edit :Chained('base') :PathPart('edit') :Args(1) {
    my ($self, $c, $location_id) = @_;

    $c->form({
        required => [ qw( name rank active ) ],
        optional => [ qw( subname parent_id map_code description address phone note url submit ) ],
        defaults => {
            rank => 0,
        },
        filters  => [ qw( trim ) ],
        missing_optional_valid => 1,
    });

    my $location;
    if ( $location_id && $location_id ne 'new' ) {
        $location = $c->model('HoursToolDB::Locations')->find({ id => $location_id });
    }

    if ( hascontent($c->form->valid->{submit}) ) {

        $c->stash->{form_submitted} = 1;
        $c->stash->{params} = $c->request->params;

        unless ($c->form->has_missing || $c->form->has_invalid || $c->form->has_unknown) {

            eval {
                if ( $location ) {
                    $location->update_from_fv($c->form);
                }
                else {
                    $location = $c->model('HoursToolDB::Locations')->create_from_fv($c->form);
                    $c->flash->{results} = [ $c->loc('Location created.') ];
                    $c->redirect( $c->uri_for( $c->controller->action_for('edit'), [ $location->id ] ) );
                }
            };
            if ($@) {
                my $err = $@;
                die($err);
            }

            push @{$c->stash->{results}}, $c->loc('Location updated.');
        }

    }

    $c->stash->{location}   = $location;
    $c->stash->{parents}    = [ map { [ $_->id, $_->name ] } $c->model('HoursToolDB::Locations')->search({ id => { '!=' => $location_id ne 'new' ? $location_id : 0 } })->all ];
    $c->stash->{template}   = 'admin/locations/edit.tt';
}


sub delete :Chained('base') :PathPart('delete') :Args(1) {
    my ($self, $c, $location_id) = @_;

    my $location = $c->model('HoursToolDB::Locations')->find({ id => $location_id });
    if ( !defined($location) ) {
        die( $c->loc('Unable to find location id: ') . $location_id );
        $c->detach;
        return;
    }

    if ( $c->req->params->{do_delete} ) {
        # TODO: Check that this is chain deleting all the attached records

        $location->delete();

        return $c->redirect( $c->uri_for( $c->controller->action_for('index') ) );
    }

    $c->stash->{location} = $location;
    $c->stash->{template} = 'admin/locations/delete.tt';
}

sub view :Chained('base') :PathPart('view') :Args(1) {
    my ($self, $c, $location_id) = @_;

    my $location = $c->model('HoursToolDB::Locations')->find({ id => $location_id });
    if ( !defined($location) ) {
        die( $c->loc('Unable to find location id: ') . $location_id );
        $c->detach;
        return;
    }

    $c->stash->{location_hours} = [ $location->location_hours->search({}, { join => 'range', order_by => 'range.start_date' })->all ];
    $c->stash->{location}       = $location;
    $c->stash->{template}       = 'admin/locations/view.tt';
}


=head1 AUTHOR

Todd Holbrook

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
