package HoursToolWeb::Controller::Admin::Ranges;
use Moose;
use namespace::autoclean;

use String::Util qw(hascontent);
use DateTime::Format::ISO8601;

BEGIN { extends 'Catalyst::Controller'; }

=head1 NAME

HoursToolWeb::Controller::Admin::Ranges - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

=cut


sub base :Chained('../base') :PathPart('range') :CaptureArgs(0) {
    my ( $self, $c ) = @_;
    $c->stash->{section} = 'ranges';
}


=head2 index

=cut

sub index :Chained('base') :PathPart('') :Args(0) {
    my ( $self, $c ) = @_;

    $c->form({
            optional => [ qw( show filter apply_filter sort page ) ],
            filters  => ['trim'],
    });

    my %sort_map = (
        name        => 'LOWER(name)',
        name_d      => 'LOWER(name) DESC',
    );

    my $ranges_rs = $c->model('HoursToolDB::Ranges')->search({},
        {
            order_by  => $sort_map{$c->form->valid('sort') || 'name'},
            page      => int( $c->form->valid('page') || 1 ),
            rows      => 30,
        }
    );

    if ( my $filter = $c->form->valid('filter') ) {
        $filter =~ s/[%_]/\\$1/g;
        $ranges_rs = $ranges_rs->search({ name => { 'ilike' => "%${filter}%" } });
    }

    $c->stash->{page}         = $c->req->params->{page};
    $c->stash->{sort}         = $c->req->params->{sort} || 'name';
    $c->stash->{filter}       = $c->req->params->{filter};
    $c->stash->{ranges_rs} = $ranges_rs;
    $c->stash->{template}     = 'admin/ranges/list.tt';
}

sub _range_constraint {
    my ( $attrs ) = @_;
    my ( $start, $end ) = @{ $attrs->{fields} } if $attrs->{fields};
    return sub {
        my $dfv        = shift;
        my $data       = $dfv->get_filtered_data;
        my $start_data = $data->{$start};
        my $end_data   = $data->{$end};

        if ( $start_data !~ /^\d{4}-\d{2}-\d{2}$/ ) {
            return 0;
        }
        if ( $end_data !~ /^\d{4}-\d{2}-\d{2}$/ ) {
            return 0;
        }

        my $start_date = DateTime::Format::ISO8601->parse_datetime($start_data);
        my $end_date   = DateTime::Format::ISO8601->parse_datetime($end_data);

        if ( $start_date > $end_date ) {
            return 0;
        }
        return 1;
    }
}

sub edit :Chained('base') :PathPart('edit') :Args(1) {
    my ($self, $c, $range_id) = @_;

    $c->form({
        required => [ qw( name start_date end_date active ) ],
        optional => [ qw( description submit ) ],
        filters  => [ qw( trim ) ],
        missing_optional_valid => 1,
        constraint_methods => {
            end => {
                constraint_method => _range_constraint({ fields => [qw/start_date end_date/] }),
                name => 'start_before_end',
            },
            start => {
                constraint_method => _range_constraint({ fields => [qw/start_date end_date/] }),
                name => 'start_before_end',
            },
        },
        msgs => {
            format => '%s',
            constraints => {
                start_before_end => $c->loc('Dates must be in YYYY-MM-DD format and the start must come before the end date.'),
            },
        },
    });

    my $range;
    if ( $range_id && $range_id ne 'new' ) {
        $range = $c->model('HoursToolDB::Ranges')->find({ id => $range_id });
    }

    if ( hascontent($c->form->valid->{submit}) ) {
        
        $c->stash->{form_submitted} = 1;
        $c->stash->{params} = $c->request->params;

        unless ($c->form->has_missing || $c->form->has_invalid || $c->form->has_unknown) {

            eval {
                if ( $range ) {
                    $range->update_from_fv($c->form);
                }
                else {
                    $range = $c->model('HoursToolDB::Ranges')->create_from_fv($c->form);
                    $c->flash->{results} = [ $c->loc('Range created.') ];
                    $c->redirect( $c->uri_for( $c->controller->action_for('edit'), [ $range->id ] ) );
                }
            };
            if ($@) {
                my $err = $@;
                die($err);
            }

            push @{$c->stash->{results}}, $c->loc('Range updated.');
        }

    }

    $c->stash->{range}      = $range;
    $c->stash->{parents}    = [ map { [ $_->id, $_->name ] } $c->model('HoursToolDB::Ranges')->search({ id => { '!=' => $range_id ne 'new' ? $range_id : 0 } })->all ];
    $c->stash->{template}   = 'admin/ranges/edit.tt';
}


sub delete :Chained('base') :PathPart('delete') :Args(1) {
    my ($self, $c, $range_id) = @_;
    
    my $range = $c->model('HoursToolDB::Ranges')->find({ id => $range_id });
    if ( !defined($range) ) {
        die( $c->loc('Unable to find range id: ') . $range_id );
        $c->detach;
        return;
    }

    if ( $c->req->params->{do_delete} ) {
        # TODO: Check that this is chain deleting all the attached records
        
        $range->delete();
        
        return $c->redirect( $c->uri_for( $c->controller->action_for('index') ) );
    }

    $c->stash->{range} = $range;
    $c->stash->{template} = 'admin/ranges/delete.tt';
}

sub view :Chained('base') :PathPart('view') :Args(1) {
    my ($self, $c, $range_id) = @_;
    
    my $range = $c->model('HoursToolDB::Ranges')->find({ id => $range_id });
    if ( !defined($range) ) {
        die( $c->loc('Unable to find range id: ') . $range_id );
        $c->detach;
        return;
    }

    $c->stash->{location_hours} = [ $range->location_hours->search({}, { join => 'location', order_by => 'LOWER(location.name)' })->all ];
    $c->stash->{range} = $range;
    $c->stash->{template} = 'admin/ranges/view.tt';
}

=head1 AUTHOR

Todd Holbrook

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
