package HoursToolWeb::Controller::Root;
use Moose;
use namespace::autoclean;

use String::Util qw(hascontent);
use Time::Simple;

BEGIN { extends 'Catalyst::Controller' }

#
# Sets the actions in this controller to be registered with no prefix
# so they function identically to actions created in MyApp.pm
#
__PACKAGE__->config(namespace => '');

=head1 NAME

HoursToolWeb::Controller::Root - Root Controller for HoursToolWeb

=head1 DESCRIPTION

[enter your description here]

=head1 METHODS

=head2 index

The root page (/)

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;

    my $now = DateTime->now;

    my $date = $c->request->param('date');
    my $dt = defined($date) ? DateTime::Format::ISO8601->parse_datetime( $date ) : $now;

    if ( $dt->ymd eq $now->ymd ) {
        $c->stash->{now}  = $now;
        $c->stash->{time} = Time::Simple->new();
    }

    my $locations = $c->model('HoursToolDB')->get_nested_locations();

    my %l;
    if ( my $l = $c->request->parameters->{l} ) {
        %l = map { $_ => 1 } ref($l) eq 'ARRAY' ? @$l : ($l);
        @$locations = grep { $l{$_->id} } @$locations;
    }

    # Build list of hours per location for use in a table display of upcoming
    # hours. Only show hours in the next six months.

    my %location_ranges;
    foreach my $location (@$locations) {
        $location_ranges{$location->id} = $location->hours_in_range( $now, DateTime->now->add( months => 6 ) );
    }

    $c->stash->{hours}      = $c->model('HoursToolDB')->hours_for_date( $dt );
    $c->stash->{locations}  = $locations;
    $c->stash->{ranges}     = \%location_ranges;
    $c->stash->{date}       = $dt;
    $c->stash->{template}   = 'index.tt';
}

sub json :Chained('/') :PathPart('json') :Args(0) {
    my ( $self, $c ) = @_;

    my $now        = DateTime->now( time_zone => 'America/Vancouver' );
    my $date       = $c->request->param('date');
    my $dt         = defined($date) ? DateTime::Format::ISO8601->parse_datetime( $date ) : $now;
    my $time       = Time::Simple->new();
    my $is_today   = $dt->ymd eq $now->ymd;

    my $hours      = $c->model('HoursToolDB')->hours_for_date( $dt );
    my $locations  = $c->model('HoursToolDB')->get_nested_locations();

    my %l;
    if ( my $l = $c->request->parameters->{l} ) {
        %l = map { $_ => 1 } ref($l) eq 'ARRAY' ? @$l : ($l);
        @$locations = grep { $l{$_->id} } @$locations;
    }

    # Build list of hours per location for use in a table display of upcoming
    # hours. Only show hours in the next six months.

    my %location_ranges;
    foreach my $location (@$locations) {
        my @location_details;
        my $lrs = $location->hours_in_range( $now, DateTime->now->add( months => 6 ) );
        foreach my $lr ( @$lrs ) {

            my $lr_hash = $lr->to_json_hash;
            $lr_hash->{start_date}  = $lr->range->start_date->ymd;
            $lr_hash->{end_date}    = $lr->range->end_date->ymd;
            $lr_hash->{name}        = $lr->range->name;
            $lr_hash->{description} = $lr->range->description;
            $lr_hash->{days}        = {};
            $lr_hash->{dows}        = $lr->range->dows;

            foreach my $day ( 1 .. 7 ) {
                my $hours = $lr->day($day);

                $lr_hash->{days}->{$day} = {
                    closed_all_day  => $hours->closed_all_day ? 1 : 0,
                    open_time       => defined($hours->open_time)  ? $hours->open_time->format('%l:%M%p')  : undef,
                    close_time      => defined($hours->close_time) ? $hours->close_time->format('%l:%M%p') : undef,
                    open_all_day    => (defined($hours->open_time) && defined($hours->close_time) && ($hours->open_time cmp '00:00:00') == 0 && ($hours->close_time cmp '00:00:00') == 0 ) ? 1 : 0,
                };
            }

            push @location_details, $lr_hash;
        }

        $location_ranges{$location->id} = \@location_details;
    }

    my $results = {
        date     => $dt->ymd,
        year     => $dt->year,
        month    => $dt->month,
        day      => $dt->day,
        next     => $dt->clone->add( days => 1 )->ymd,
        prev     => $dt->clone->add( days => -1 )->ymd,
        today    => $now->ymd,
        is_today => $is_today,
    };

    if ( $is_today ) {
        $results->{now}  = $now->ymd;
        $results->{time} = $time->format('%l:%M%p');
    }

    foreach my $hour ( values %$hours ) {
        $hour->{open_all_day} = ( defined($hour->{open_time}) && defined($hour->{close_time}) && ($hour->{open_time} cmp '00:00:00') == 0 && ($hour->{close_time} cmp '00:00:00') == 0 ) ? 1 : 0;
        $hour->{in_range}     = ( $is_today && ( $hour->{closed_all_day} || ($time >= $hour->{open_time} && $time <= $hour->{close_time}) ) ) ? 1 : 0;
        $hour->{close_time}   = defined($hour->{close_time}) ? $hour->{close_time}->format('%l:%M%p') : undef;
        $hour->{open_time}    = defined($hour->{open_time})  ? $hour->{open_time}->format('%l:%M%p')  : undef;
    }


    $results->{ranges}    = \%location_ranges;

    $results->{locations} = [ map { $_->to_json_hash } @$locations ];
    $results->{hours}     = $hours;

    $c->stash->{current_view} = 'JSON';
    $c->stash->{json} = $results;
}

=head2 default

Standard 404 error page

=cut

sub default :Path {
    my ( $self, $c ) = @_;
    $c->response->body( 'Page not found' );
    $c->response->status(404);
}

=head2 end


Attempt to render a view, if needed.

=cut

sub end : ActionClass('RenderView') {
	my ( $self, $c ) = @_;
    if ( !hascontent($c->stash->{results}) && hascontent($c->flash->{results}) ) {
        $c->stash->{results} = $c->flash->{results};
    }

}

=head1 AUTHOR

Todd Holbrook

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
