package HoursToolWeb::Model::HoursToolDB;

use strict;
use base 'Catalyst::Model::DBIC::Schema';
use DateTime::Format::ISO8601;


__PACKAGE__->config(
    schema_class => 'HoursTool::Schema',  
);


sub hours_for_date {
	my ( $self, $date ) = @_;

    my $dt = $date->isa('DateTime') ? $date : DateTime::Format::ISO8601->parse_datetime( $date );
    if ( !defined($dt) ) {
    	die( 'Unable to parse date: ' . $date );
    }

    my $hfd_rs = $self->schema->resultset('HoursForDay')->search_day( $dt->ymd );

    my %results;
    my %hfds;
	while ( my $hfd = $hfd_rs->next ) {
		next if exists $results{$hfd->location_id}; # Optimize this out later by GROUP/DISTINCT on the database call
		$hfds{$hfd->location_id} = $hfd;
	}

	foreach my $location_id ( keys %hfds ) {

		my $hfd    = $hfds{$location_id};
		my $record = $hfd->to_json_hash;

		my $parent = $hfd->parent;
		my $range  = $hfd->range;
		if ( defined($parent) ) {
			my $parent_hfd = $hfds{$parent->id};
			if ( !defined($range) || $parent_hfd->priority > $hfd->priority ) {
				$hfd = $parent_hfd;
				$range = $hfd->range;
			}
		}

		next if !defined($range);

		$record->{range} = $range->to_json_hash;

		my $day = $hfd->day( $dt->dow );
		if ( defined($day) ) {
			$record->{open_time}             = $day->open_time;
			$record->{close_time}            = $day->close_time;
			$record->{closed_all_day}  		 = !!$day->closed_all_day;
		}

		$results{ $location_id } = $record;
	}

	return \%results;
}


sub get_nested_locations {
	my $self = shift;

	my %locations = map { $_->id => $_ } $self->schema->resultset('Locations')->search({ active => 't' })->all;

	# Nest them by parent. We're only going one deep at this point. There's surely a nicer way to do this, but the
	# numbers are small enough it doesn't seem worth optimizing yet

	foreach my $id ( keys %locations ) {
		my $parent_id = $locations{$id}->parent_id;
		next if !defined $parent_id;

		$locations{ $parent_id }->{children} if !exists $locations{ $parent_id }->{children};
		push @{$locations{ $parent_id }->{children}}, delete $locations{$id};
	}

	# Flatten them for easy looping

	my @locations;
	foreach my $location ( sort { $b->rank <=> $a->rank || $a->name cmp $b->name } values %locations ) {
		push @locations, $location;
		if ( $location->{children} ) {
			push @locations, sort { $b->rank <=> $a->rank || $a->name cmp $b->name } @{$location->{children}};
		}
	}

	return \@locations;
}

=head1 NAME

HoursToolWeb::Model::HoursToolDB - Catalyst DBIC Schema Model

=head1 SYNOPSIS

See L<HoursToolWeb>

=head1 DESCRIPTION

L<Catalyst::Model::DBIC::Schema> Model using schema L<HoursTool::Schema>

=head1 GENERATED BY

Catalyst::Helper::Model::DBIC::Schema - 0.59

=head1 AUTHOR

Todd Holbrook

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
